/*
 * nasd_cheops_cl_vnasd.h
 */
/*
 * Cheops client clerk service module exporting a virtual NASD interface
 *
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE, July 18 1997
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */


#include <nasd/nasd_options.h>
#include <stdio.h>
#include <dce/dce_error.h>
#include <sys/errno.h>

#include <nasd/nasd_types.h>
#include <nasd/nasd_cheops_types.h>
#include <nasd/nasd_cheops_cache.h>
#include <nasd/nasd_cheops_common.h>
#include <nasd/nasd_cheops_raidmap.h>

/* clerk initialization functions */

extern int
_nasd_cheops_cl_init();

/* the cheops clerk (virtual NASD) interface */

extern void 
_nasd_null_cl (handle_t       client_handle, 
	      nasd_status_t  *nasd_status,
	      error_status_t *op_status);

extern void 
_nasd_getattr_cl (handle_t           client_handle,
		 nasd_cookie_t      in_cookie,
		 nasd_identifier_t  in_identifier,
		 nasd_partnum_t      in_partnum,
		 nasd_attribute_t   *out_attribute,
		 nasd_status_t      *nasd_status,
		 error_status_t     *op_status);
extern void 
_nasd_setattr_cl (handle_t           client_handle,
		 nasd_cookie_t      in_cookie,
		 nasd_identifier_t  in_identifier,
		 nasd_attribute_t   *in_attribute,
		 nasd_fieldmask_t   in_fieldmask,
		 nasd_partnum_t     in_partnum,
		 nasd_guard_t       in_guard,
		 nasd_attribute_t   *out_attribute,
		 nasd_status_t      *nasd_status,
		 error_status_t     *op_status);
extern void
_nasd_write_cl (handle_t            client_handle,
	       nasd_cookie_t        in_cookie,
	       nasd_identifier_t    in_identifier,
	       nasd_offset_t        in_offset,
	       nasd_len_t           in_len,
	       nasd_partnum_t       in_partnum,
	       nasd_len_t           *out_len,
	       nasd_byte_t          *in_data,
	       nasd_status_t        *nasd_status,
	       error_status_t       *op_status);


extern void 
_nasd_read_cl (handle_t             client_handle,
	       nasd_cookie_t        in_cookie,
	       nasd_identifier_t    in_identifier,
	       nasd_offset_t        in_offset,
	       nasd_len_t           in_len,
	       nasd_partnum_t       in_partnum,
	       nasd_len_t           *out_len,
	       nasd_byte_t          *out_data,
	       nasd_status_t        *nasd_status,
	       error_status_t       *op_status);


extern void 
_nasd_access_cl (handle_t            client_handle,
		nasd_cookie_t       in_cookie,
		nasd_identifier_t   in_identifier,
		nasd_cookie_t       *out_capability, /* FIX -- */
		nasd_status_t       *nasd_status,
		error_status_t      *op_status);

extern void
_nasd_part_create_cl (handle_t      client_handle,
		     nasd_cookie_t   in_cookie, 
		     nasd_partnum_t  in_partnum,  
		     nasd_uint64     in_blkcnt,
		     nasd_status_t   *nasd_status,
		     error_status_t  *op_status);
extern void 
_nasd_qos_create_cl (handle_t            client_handle,
		    nasd_cookie_t       in_cookie,
		    nasd_attribute_t    in_attribute,
		    nasd_fieldmask_t    in_fieldmask,
		    nasd_partnum_t      in_partnum,
		    _nasd_cheops_qos_req_t in_qos_req,
		    nasd_identifier_t   *out_identifier,
		    nasd_attribute_t    *out_attribute,
		    nasd_status_t       *nasd_status,
		    error_status_t      *op_status);

extern void 
_nasd_create_cl (handle_t            client_handle,
		nasd_cookie_t       in_cookie,
		nasd_attribute_t    in_attribute,
		nasd_fieldmask_t    in_fieldmask,
		nasd_partnum_t      in_partnum,
		nasd_identifier_t   *out_identifier,
		nasd_attribute_t    *out_attribute,
		nasd_status_t       *nasd_status,
		error_status_t      *op_status);
extern void 
_nasd_remove_cl (handle_t            client_handle,
		nasd_cookie_t       in_cookie,
		nasd_identifier_t   in_identifier,
		nasd_status_t       *nasd_status,
		error_status_t      *op_status);

/*
 * Get static configuration info
 */
extern void 
_nasd_getinfo_cl(handle_t            client_handle,
		nasd_cookie_t       in_cookie,
		nasd_drive_info_t   *out_info,
		nasd_status_t       *nasd_status,
		error_status_t      *status);

/*
 * Get dynamic configuration info
 */
extern void 
_nasd_getstatus_cl(handle_t             client_handle,
		  nasd_cookie_t        in_cookie,
		  nasd_drive_status_t  *out_status,
		  nasd_status_t        *nasd_status,
		  error_status_t       *status);










