/*
 * nasd_cheops_time.h
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_timer.h>

#define CHEOPS_REPORT_BANDWIDTH(_mb_) \
 (cheops_Msg "Bandwidth %f MB/sec\n", (_mb_)/accum_time)

#define CHEOPS_DECLARE_TIME \
  double accum_time, d, avg; \
  nasd_timespec_t diff; \
  nasd_timer_t timer; \
  int iter;

#define CHEOPS_INIT_TIME \
  accum_time = (double)0.0;

#define CHEOPS_BEGIN_TIME \
  NASD_TM_START(&timer); \

#define CHEOPS_END_TIME \
  NASD_TM_STOP(&timer); \
  NASD_TM_ELAPSED_TS(&timer, &diff); \
  d = (double)diff.ts_nsec; \
  d /= (double)1000000000.0; \
  d += (double)diff.ts_sec; \
  accum_time += d;

#define CHEOPS_REPORT_TIME(_op_) { \
  iter = 1; \
  avg = accum_time; \
  printf("OPERATION: %s\n", _op_); \
  printf("  %f seconds for %d operations\n", accum_time, iter); \
  printf("  %f seconds average\n", avg); \
}
