/*
 * nasd_cheops_cl_drive.h
 */
/*
 * cheops drive access routines
 *
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE, July 18 1997
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
#ifndef _NASD_CHEOPS_CL_DRIVE_H_
#define _NASD_CHEOPS_CL_DRIVE_H_

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <dce/dce_error.h>
#include <nasd/nasd_pdrive.h>
#include <nasd/nasd_pdrive_client.h>
#include <nasd/nasd_general.h>

#include <nasd/nasd_cheops_types.h>
#include <nasd/nasd_cheops_common.h>
#include <nasd/nasd_cheops_cache.h>
#include <nasd/nasd_cheops_mgr_structs.h>
#include <nasd/nasd_cheops_mgr_common.h>

void
report_err(nasd_status_t nasd_status, unsigned32 status, char *op);

int
_nasd_cheops_null_dr(int di, nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_null_h_dr(nasd_drive_handle_t h, nasd_status_t *nasd_status, nasd_rpc_status_t *status);

/* XXXmju: this does not seem to be defined anywhere in the Cheops source. */
int
_nasd_cheops_part_create_dr(int di, nasd_partnum_t in_partnum, 
			    nasd_key_t in_key, int in_nblocks,
			    nasd_status_t *nasd_status, nasd_rpc_status_t *status);
int
_nasd_cheops_part_create_h_dr(nasd_drive_handle_t h, nasd_partnum_t in_partnum, 
			      nasd_key_t in_key, int in_nblocks,
			      nasd_status_t *nasd_status, nasd_rpc_status_t *status);
int
_nasd_cheops_create_dr(int di, nasd_partnum_t in_partnum,
		      nasd_key_t req_key, nasd_identifier_t *out_identifier,
		      nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_create_h_dr(nasd_drive_handle_t h, nasd_partnum_t in_partnum,
		      nasd_key_t req_key, nasd_identifier_t *out_identifier,
		      nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int 
_nasd_cheops_remove_dr(int di,  nasd_partnum_t partnum, 
		      nasd_key_t req_key, nasd_identifier_t in_identifier,
		      nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int 
_nasd_cheops_remove_h_dr(nasd_drive_handle_t h,  nasd_partnum_t partnum, 
		      nasd_key_t req_key, nasd_identifier_t in_identifier,
		      nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int 
_nasd_cheops_getattr_dr(int di, nasd_partnum_t partnum, 
		       nasd_key_t req_key, nasd_identifier_t in_id, 
		       nasd_attribute_t *out_attribute,
		       nasd_status_t *nasd_status, nasd_rpc_status_t *status);
int 
_nasd_cheops_getattr_h_dr(nasd_drive_handle_t h, nasd_partnum_t partnum, 
		       nasd_key_t req_key, nasd_identifier_t in_id, 
		       nasd_attribute_t *out_attribute,
		       nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_setattr_dr(int di, nasd_partnum_t partnum, 
		       nasd_key_t req_key, nasd_identifier_t in_identifier, 
		       nasd_attribute_t *in_attribute,
		       nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_setattr_h_dr(nasd_drive_handle_t h, nasd_partnum_t partnum, 
			  nasd_key_t req_key, nasd_identifier_t in_identifier, 
			  nasd_attribute_t *in_attribute,
			  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_read_dr(int di, nasd_partnum_t in_partnum, 
		    nasd_key_t req_key, nasd_identifier_t in_identifier,
		    nasd_byte_t *buf, nasd_offset_t in_offset, nasd_len_t in_len, 
		    nasd_uint64 bms_targ, nasd_len_t *out_len,
		    nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_write_dr(int di, nasd_partnum_t in_partnum, 
		     nasd_key_t req_key, nasd_identifier_t in_identifier,
		     nasd_byte_t *buf, nasd_offset_t in_offset, nasd_len_t in_len, 
		     nasd_len_t *out_len, nasd_status_t *nasd_status, nasd_rpc_status_t *status);

int
_nasd_cheops_tswrite_dr(int di, nasd_partnum_t in_partnum, 
			nasd_key_t req_key, nasd_identifier_t in_identifier,
			nasd_byte_t *buf, nasd_offset_t in_offset, nasd_len_t in_len, nasd_host_tag_t host_tag,
			nasd_len_t *out_len, nasd_status_t *nasd_status, nasd_rpc_status_t *status);

#endif /*  _NASD_CHEOPS_CL_DRIVE_H_ */
