/*
 * nasd_generic_addts.c
 *
 * Generic inc/add/subtract nasd timespec for most OSes.
 * Not very fast (involves locking/unlocking a mutex), but is portable.
 * If your architecture has atomic inc/add/sub operations, you probably
 * want to write a machine-specific version of this file.
 */

/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Jim Zelenka, Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_threadstuff.h>

NASD_DECLARE_EXTERN_MUTEX(nasd_atomic_global_mutex);

void
nasd_atomic_add_timespec(
  nasd_timespec_t  *result,
  nasd_timespec_t  *addend)
{
  NASD_LOCK_MUTEX(nasd_atomic_global_mutex);
  NASD_TIMESPEC_ADD(*result,*addend);
  NASD_UNLOCK_MUTEX(nasd_atomic_global_mutex);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
