/*
 * nasd_rpcc.c
 *
 * Read-cycle-counter ops for Pentium-family processors.
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_timer.h>
#include <time.h>
#include <unistd.h>

/* this is sort of a hack, but there is no way to ask the processor
   what its clock speed is. */
nasd_status_t
nasd_get_clock_speed(nasd_uint64 *ticksp)
{
  struct timeval stv, etv;
  nasd_uint32 spcc, epcc;
  nasd_uint32 elap, elap_cc;
  static nasd_uint64 result;

  if(!result) {
    nasd_fast_gettime(&stv);
    spcc = NASD_rpcc();
    usleep(100000);		/* 100ms */
    epcc = NASD_rpcc();
    nasd_fast_gettime(&etv);

    elap = ((etv.tv_sec - stv.tv_sec) * 1000000) + (etv.tv_usec - stv.tv_usec);
    elap_cc = epcc - spcc;
    result = ((nasd_uint64)elap_cc * 1000000) / elap;
  }

  *ticksp = result;
  return NASD_SUCCESS;
}

/* Read the Pentium timestamp counter and return the low 32 bits. */
nasd_uint32
NASD_rpcc(void)
{
  nasd_uint32 low;
  nasd_uint32 high;

  __asm__(".byte 0x0f,0x31" :"=a" (low), "=d" (high)); /* RDTSC */
  return low;
}

/* Version of NASD_rpcc that returns all 64 bits of the TSC value. */
void
NASD_rpcc_int64(nasd_uint32 *high, nasd_uint32 *low)
{
  __asm__(".byte 0x0f,0x31" :"=a" (*low), "=d" (*high)); /* RDTSC */
  return;
}
