/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Howard Gobioff
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_security_time.c
 *
 * Timekeeping functions for NASD security code
 *
 ***********************************************************************/

#include <nasd/nasd_options.h>
#if defined(_KERNEL) && defined(DEC_OSF)
#include <sys/kernel.h>
#endif /* _KERNEL && DEC_OSF */
#include <nasd/nasd_general.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_common.h>
#include <nasd/nasd_security_time.h>

nasd_timespec_t unique_time;


    
/* This code should be connected to a protocol where the drive and
   file manager agree on the time. Essentially, the filemanager will
   periodically query the drive and ask what time it is. The filemanager
   will forward this information out when it issues capabilities.*/

nasd_status_t nasd_security_get_drive_time(nasd_drive_name_t drive_name, nasd_timespec_t *now)
{
#ifdef KERNEL
  struct timeval t;
#endif /* KERNEL */
  
#if NASD_SECURE_RPCS_ENABLE > 0
#ifdef DEBUG_GET_DRIVE_TIME
  printf("DEBUG: nasd_security_get_drive_time %d\n",drive_name);
#endif /* DEBUG_GET_DRIVE_TIME */
  /* However, we will use the clock for now*/
#ifndef KERNEL
  nasd_gettime(now);
#else 
  TIME_READ(t);
  
  now->ts_sec=t.tv_sec;
  now->ts_nsec=t.tv_usec*1000;
#endif 
  
  if (now->ts_sec != unique_time.ts_sec ||
      now->ts_nsec != unique_time.ts_nsec) {
    unique_time.ts_sec = now->ts_sec;
    unique_time.ts_nsec = now->ts_nsec;
  } else {
    unique_time.ts_nsec++;
    now->ts_nsec++;
  }
  
  
  
#ifdef DEBUG_GET_DRIVE_TIME
  printf("DEBUG: nasd_security_get_drive_time returning %d.%d\n",now->ts_sec,now->ts_nsec);
#endif
  return(NASD_SUCCESS);
#else /* NASD_SECURE_RPCS_ENABLE > 0 */
  
  return(NASD_SUCCESS); /* cheap security - nice and fast */

#endif /* NASD_SECURE_RPCS_ENABLE >0 */
}

