/*
 * nasd_vx_basic.c
 *
 * Basic system support stuff
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Marc Unangst, Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_threadstuff.h>
#include <sys/time.h>

nasd_status_t
nasd_sys_init()
{
  return(NASD_SUCCESS);
}

void
nasd_gettime(
  nasd_timespec_t  *timep)
{
  struct timespec ts;
  int ret;

  ret = clock_gettime(CLOCK_REALTIME, &ts);
  NASD_ASSERT(ret == 0);
  timep->ts_sec = ts.tv_sec;
  timep->ts_nsec = ts.tv_nsec;
}

nasd_status_t
nasd_mutex_init(SEM_ID *m)
{
  *m = WIND_MUTEX_CREATE();
  if(*m)
    return NASD_SUCCESS;
  return NASD_FAIL;
}


nasd_status_t
nasd_mutex_destroy(SEM_ID *m)
{
  int rc;

  rc = WIND_MUTEX_DESTROY(*m);
  if(rc)
    return NASD_FAIL;
  return NASD_SUCCESS;
}


nasd_status_t
nasd_cond_init(SEM_ID *c)
{
  *c = WIND_COND_CREATE();
  if(*c)
    return NASD_SUCCESS;
  return NASD_FAIL;
}


nasd_status_t
nasd_cond_destroy(SEM_ID *c)
{
  int rc;

  rc = WIND_COND_DESTROY(*c);
  if(rc)
    return NASD_FAIL;
  return NASD_SUCCESS;
}

nasd_status_t
nasd_sys_threadattr_create(
  nasd_sys_threadattr_t  *attrp)
{
  attrp->name = NULL;
  attrp->priority = NASD_WIND_DEF_PRIORITY;
  attrp->options = NASD_WIND_DEF_OPTIONS;
  attrp->stackSize = NASD_WIND_DEF_STACKSIZE;

  return(NASD_SUCCESS);
}

nasd_status_t
nasd_sys_threadattr_destroy(
  nasd_sys_threadattr_t  *attrp)
{
  return(NASD_SUCCESS);
}

nasd_status_t
nasd_sys_threadattr_setstacksize(
  nasd_sys_threadattr_t  *attrp,
  int                     size)
{
  attrp->stackSize = size;
  return(NASD_SUCCESS);
}

nasd_status_t
nasd_sys_threadattr_getstacksize(
  nasd_sys_threadattr_t  *attrp,
  int                    *sizep)
{
  *sizep = attrp->stackSize;
  return(NASD_SUCCESS);
}

nasd_status_t
nasd_sys_thread_create_attr(
  nasd_sys_thread_t      *threadp,
  nasd_sys_threadattr_t  *attrp,
  nasd_threadfunc_t       func,
  nasd_threadarg_t        arg)
{
  *threadp = taskSpawn(attrp->name, attrp->priority, attrp->options,
    attrp->stackSize, func, arg,
    0, 0, 0, 0, 0, 0, 0, 0, 0);
  if (*threadp == NULL)
    return(NASD_FAIL);
  return(NASD_SUCCESS);
}

nasd_status_t
nasd_sys_thread_create(
  nasd_sys_thread_t  *threadp,
  nasd_threadfunc_t   func,
  nasd_threadarg_t    arg)
{
  *threadp = taskSpawn("anonymous", NASD_WIND_DEF_PRIORITY,
    NASD_WIND_DEF_OPTIONS, NASD_WIND_DEF_STACKSIZE,
    func, arg,
    0, 0, 0, 0, 0, 0, 0, 0, 0);
  if (*threadp == NULL)
    return(NASD_FAIL);
  return(NASD_SUCCESS);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
