/*
 * decompose.c
 *
 * Decompose a NASD identifier.
 */
/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_freelist.h>
#include <nasd/nasd_itypes.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_cache.h>
#include <nasd/nasd_common.h>

char *progname;

void
usage()
{
  fprintf(stderr, "USAGE: %s nasd_identifier\n", progname);
  fflush(stderr);
  exit(1);
}

int
main(
  int     argc,
  char  **argv)
{
  nasd_nodenum_t nodenum;
  nasd_identifier_t nid;
  nasd_generation_t gen;
  nasd_blkno_t hint;
  nasd_status_t rc;
  int partnum;

  progname = argv[0];

  if (argc != 2)
    usage();

  rc = nasd_str_to_nasd_id(argv[1], &nid);
  if (rc)
    usage();

  rc = nasd_od_decompose_id(nid, &partnum, &nodenum, &hint, &gen);
  if (rc) {
    fprintf(stderr, "ERROR: got status=%lu (%s) from nasd_od_decompose_id()\n",
      (unsigned long)rc, nasd_error_string(rc));
    fflush(stderr);
    exit(1);
  }

  printf("Partition %d  Node %d  Hinted block %u  Generation %d\n",
    (int)partnum, (int)nodenum, (unsigned int)hint, (int)gen);
  fflush(stdout);

  exit(0);
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
