/*
 * nasd_drive_tunable.c
 *
 * Global declarations of tunable drive parameters.
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_drive_options.h>
#include <nasd/nasd_cache.h>

/*
 * What percentage of the disk might be inodes
 */
int nasd_od_nnpt_pcg = 75;

/*
 * When prefetching regions, stop clustering forward or backward if
 * a block is unused (physical refcount = 0).
 */
int nasd_odc_pfcontig_only = 0;

/*
 * When prefetching regions, be willing to fetch useless blocks
 */
int nasd_odc_pfcontig_useless = 1;

/*
 * When prefetching regions, abort clustering when we hit an
 * in-core block (avoid skipping)
 */
int nasd_odc_pfcontig_noskip = 1;

/*
 * When prefetching, override above settings to allow skipping
 * after the fetch block.
 *
 * 0 = no override
 * 1 = skip past incore blocks
 * 2 = skip past incore blocks, skip over noref blocks
 */
int nasd_odc_pfcontig_afterskip_policy = 2;

/*
 * Read whole regions at once.
 *
 * !!! Reassigned by most layout policy modules !!!
 */
int nasd_odc_read_regions = 0;

/*
 * Number of blocks in cache.
 *
 * !!! Resassigned by drive front-end !!!
 */
int nasd_odc_size = 1024;

/*
 * Number of buckets in cache hash table
 */
int nasd_odc_buckets = 127;

/*
 * Do we force alignment of the first datablock?
 * If so, to what boundary? (0 = no force)
 * This is a sector count, so 64=32k, 128=64k, etc
 */
int nasd_od_force_align_disk_boundary = 256;

/*
 * How many sectors reserved for system overhead
 */
nasd_sectcnt_t nasd_overhead_sects = 1;

/*
 * How many blocks are in a region.
 * (Multiple subsystems are interested in this;
 * layout policies which use regions are obligated
 * to use this as the size of those regions.)
 */
int nasd_od_region_blocks = 16;

/*
 * How many blocks to map at a time for read or write
 *
 * This may have any value in the range 1..MAX_BLOCKMAPCHUNK
 * It should *not* be patched while the drive is dispatching
 * I/Os. Someone should do some studies sometime to see how
 * diddling this affects performance.
 */
int nasd_od_obj_blockmapchunk = 32;

/*
 * Chunksize for hashing hints for list of unallocated blocks
 */
int nasd_drive_unallocated_hash_chunk = 800;

/*
 * Max blocks to coalesce into one I/O
 */
int nasd_drive_io_max_coalesce = 8;

/*
 * Similarly, prevent I/Os from crossing some alignment boundary.
 * This is useful if running on something like a striper or RAIDframe.
 * This is in terms of sectors (64=32k, 128=64k, ...).
 */
int nasd_drive_io_cluster_align = 256;

nasd_uint64 nasd_od_shutdown_timeouts_max = 10;
nasd_timespec_t nasd_od_shutdown_timeout_interval = {1, 0};

#if NASD_TRACE_DR > 0
int nasd_dt_max_trace_size = 1048576;
#endif /* NASD_TRACE_DR > 0 */

#if defined(DEC_OSF) && defined(KERNEL)
int nasd_k_flush_check_milliseconds = 2000; /* how often to check */
int nasd_k_flush_interval           = 3;    /* how many checks before force */
int nasd_k_dirty_percent            = 30;   /* % can be dirty before flush */

#endif /* defined(DEC_OSF) && defined(KERNEL) */

#ifndef KERNEL
int nasd_od_rusage_at_shutdown = 0;
#endif /* !KERNEL */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
