/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Howard Gobioff
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_keymgmt.c
 *
 * Routines to access keys 
 * Use by both the file manager and the drive
 *
 ***********************************************************************/
#define PRINT_ERROR 1
#ifdef DEBUG_ALL
#define DEBUG_INIT_KEY_MGMT 1
#endif

#include <nasd/nasd_options.h>
#include <nasd/nasd_drive_options.h>
#include <nasd/nasd_keymgmt.h>
#include <nasd/nasd_threadstuff.h>
#include <nasd/nasd_security_dr.h>
#include <nasd/nasd_cache.h>

#ifndef KERNEL
#include <string.h>
#endif /* !KERNEL */


/* Our lock on the key hiearchy - grab it when you
   change or read a lock*/
NASD_DECLARE_MUTEX(keylock)

nasd_status_t nasd_security_init_key_mgmt(char *keyfile,char *ignored_pwd,int ignored_int){
#if NASD_SECURE_RPCS_ENABLE > 0
#if  DEBUG_INIT_KEY_MGMT > 0
  printf("Initializing Keylock \n");
#endif
  nasd_mutex_init(&keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/

  return NASD_SUCCESS;
}

nasd_status_t nasd_security_get_red_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *)PART(partnum).red_key, (char *)target, sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}



nasd_status_t nasd_security_get_black_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *)PART(partnum).black_key, (char *)target, sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}


nasd_status_t nasd_security_get_partition_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *) PART(partnum).partition_key, (char *)target, 
        sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}

nasd_status_t  nasd_security_get_master_key(nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
 NASD_LOCK_MUTEX(keylock); 
 bcopy((char *)DISK.master_key, (char *)target, sizeof(nasd_key_t));
 NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
return NASD_SUCCESS;
}


nasd_status_t  nasd_security_get_drive_key(nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
 NASD_LOCK_MUTEX(keylock); 
 bcopy((char *) DISK.drive_key, (char *)target, sizeof(nasd_key_t));
 NASD_UNLOCK_MUTEX(keylock);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
return NASD_SUCCESS;
}






nasd_status_t nasd_security_set_red_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *) target, (char *)PART(partnum).red_key, sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
  nasd_od_write_diskstate(0);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}



nasd_status_t nasd_security_set_black_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *)target, (char *)PART(partnum).black_key,  sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
  nasd_od_write_diskstate(0);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}


nasd_status_t nasd_security_set_partition_key(nasd_partnum_t partnum, nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
  NASD_LOCK_MUTEX(keylock);
  bcopy((char *)target, (char *)PART(partnum).partition_key, sizeof(nasd_key_t));
  NASD_UNLOCK_MUTEX(keylock);
  nasd_od_write_diskstate(0);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
  return NASD_SUCCESS;
}



nasd_status_t  nasd_security_set_drive_key(nasd_key_t target){
#if NASD_SECURE_RPCS_ENABLE > 0
 NASD_LOCK_MUTEX(keylock); 
 bcopy((char *)target, (char *)DISK.drive_key, sizeof(nasd_key_t));
 NASD_UNLOCK_MUTEX(keylock);
 nasd_od_write_diskstate(0);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
return NASD_SUCCESS;
}


nasd_status_t  nasd_security_initialize_drive(nasd_key_t master_key, nasd_key_t drive_key){
nasd_status_t rc=NASD_SUCCESS;
#if NASD_SECURE_RPCS_ENABLE > 0

 NASD_LOCK_MUTEX(keylock);
 if (DISK.initialized) {
   rc=NASD_DRIVE_INITIALIZED;
 } else {


   bcopy(drive_key,DISK.drive_key,sizeof(nasd_key_t));
   bcopy(master_key,DISK.master_key,sizeof(nasd_key_t));
   DISK.initialized=1;

 }
 NASD_UNLOCK_MUTEX(keylock);
 nasd_od_write_diskstate(1);
#endif /* NASD_SECURE_RPCS_ENABLE >0*/
return rc;
}


nasd_status_t nasd_security_change_key_drive(nasd_uint16 type,nasd_key_t  newkey, nasd_partnum_t partnum) {
  if  (type==NASD_MASTER_KEY) {
    return (NASD_IMMUTABLE_KEY);
  }
  else if (type== NASD_DRIVE_KEY) {
    NASD_LOCK_MUTEX(keylock);
    bcopy((char *)newkey, (char *)DISK.drive_key, sizeof(nasd_key_t));
    NASD_UNLOCK_MUTEX(keylock);
  } else if (!(NASD_OD_PARTNUM_VALID(partnum))) {
    return(NASD_BAD_PARTITION);
  } else if (type== NASD_PARTITION_KEY) {
    NASD_LOCK_MUTEX(keylock);
    bcopy((char *)newkey, (char *)PART(partnum).partition_key, 
          sizeof(nasd_key_t));
    NASD_UNLOCK_MUTEX(keylock);
  } else if (type== NASD_RED_KEY) {
    NASD_LOCK_MUTEX(keylock);
    bcopy((char *)newkey, (char *)PART(partnum).red_key, sizeof(nasd_key_t));
    NASD_UNLOCK_MUTEX(keylock);
  } else if (type== NASD_BLACK_KEY) {
    NASD_LOCK_MUTEX(keylock);
    bcopy((char *)newkey, (char *)PART(partnum).red_key, sizeof(nasd_key_t));
    NASD_UNLOCK_MUTEX(keylock);
  }
  return(NASD_SUCCESS);
}






