/*
 * nasd_od_uio.c
 *
 * I/O for vxworks NASD.
 */

/*
 * Copyright (c) 1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Jim Zelenka, Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_drive_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_freelist.h>
#include <nasd/nasd_itypes.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_cache.h>
#include <nasd/nasd_common.h>
#include <nasd/nasd_timer.h>
#include <nasd/nasd_control.h>
#define NASD_DRIVE_IO_MODULE 1
#include <nasd/nasd_drive_io.h>

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <vxWorks.h>
#include <semLib.h>

SEM_ID nasd_u_flushsem;

void
nasd_od_uio_flush_proc(
  nasd_threadarg_t  ignored)
{
  nasd_status_t rc;
  int ret;
  char c;
  int tick_delay;

  NASD_IO_PEND_LOCK();
  nasd_u_flushproc_running = 1;
  NASD_IO_PEND_UNLOCK();
  NASD_BROADCAST_COND(nasd_u_run_cond);

  /* hopefully sysClkRate doesn't change (much) */
  tick_delay = sysClkRateGet() * 5;

  while(1) {
    ret = semTake(nasd_u_flushsem, tick_delay);
    if(ret) {
      NASD_ASSERT(errno == S_objlib_OBJ_TIMEOUT);
    }
    NASD_IO_INC_STAT(auto_flush);
    rc = nasd_odc_flush_dirty(0);
    NASD_ASSERT(rc == NASD_SUCCESS);
  }
  NASD_IO_PEND_LOCK();
  nasd_u_flushproc_running = 0;
  NASD_BROADCAST_COND(nasd_u_run_cond);
  NASD_IO_PEND_UNLOCK();
}

void
nasd_u_stop_flush_proc(
  void  *ignored)
{
  int ret;

  nasd_odc_dirty_kick();
  NASD_THREADGROUP_INDICATE_SHUTDOWN(&nasd_u_flushproc_threadgroup);
  nasd_odc_dirtythread_force = 1;

  ret = semGive(nasd_u_flushsem);
  if (ret) {
    NASD_PANIC();
  }

#if MJU_DEBUG
  fprintf(stderr, "stop_flush_proc: wait for shutdown\n");
#endif /* MJU_DEBUG */
  NASD_THREADGROUP_WAIT_STOP(&nasd_u_flushproc_threadgroup);
#if MJU_DEBUG
  fprintf(stderr, "stop_flush_proc: done waiting for shutdown\n");
#endif /* MJU_DEBUG */
}

nasd_status_t
nasd_od_io_init(
  char  *devname)
{
  nasd_u_flushsem = semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
  if(!nasd_u_flushsem) {
    printf("ERROR %d from semBCreate()\n", errno);
    return NASD_FAIL;
  }
}

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
