/*
 * nasd_dux_threads_kernel.h
 *
 * Inside-dux-kernel variant of threads portability layer
 */
/*
 * Copyright (c) 1995,1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: Mark Holland, Daniel Stodolsky, Jim Zelenka, Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_DUX_THREADS_KERNEL_H_
#define _NASD__NASD_DUX_THREADS_KERNEL_H_

#include <nasd/nasd_options.h>
#include <kern/task.h>
#include <kern/thread.h>
#include <kern/lock.h>
#include <kern/sched_prim.h>

extern task_t first_task;

typedef struct nasd_once_s nasd_once_t;
struct nasd_once_s {
  int  done;
};

#define NASD_DECLARE_ONCE(_o_)                nasd_once_t _o_;

#define NASD_SYS_DECLARE_MUTEX(_m_)           decl_simple_lock_data(,(_m_))
#define NASD_SYS_DECLARE_STATIC_MUTEX(_m_)    decl_simple_lock_data(static,(_m_))
#define NASD_SYS_DECLARE_EXTERN_MUTEX(_m_)    decl_simple_lock_data(extern,(_m_))
#define NASD_SYS_DECLARE_COND(_c_)            int _c_;
#define NASD_SYS_DECLARE_STATIC_COND(_c_)     static int _c_;
#define NASD_SYS_DECLARE_EXTERN_COND(_c_)     extern int _c_;

#define NASD_SYS_LOCK_MUTEX(_m_)              simple_lock(&(_m_))
#define NASD_SYS_TRY_LOCK_MUTEX(_m_)          simple_lock_try(&(_m_))
#define NASD_SYS_UNLOCK_MUTEX(_m_)            simple_unlock(&(_m_))

extern int _nasd_once(nasd_once_t *once_block,
  void (*init_routine)(void), char *file, int line);
#define nasd_once(_o_,_i_) _nasd_once(_o_, (void (*)(void))_i_, __FILE__, __LINE__)

#define NASD_SYS_WAIT_COND(_c_,_m_) { \
  assert_wait((vm_offset_t)&(_c_), TRUE); \
  NASD_UNLOCK_MUTEX(_m_); \
  thread_block(); \
  NASD_LOCK_MUTEX(_m_); \
}
#define NASD_SYS_SIGNAL_COND(_c_) \
  thread_wakeup_one(((vm_offset_t)&(_c_)))
#define NASD_SYS_BROADCAST_COND(_c_) \
  thread_wakeup(((vm_offset_t)&(_c_)))

typedef unsigned long       nasd_thread_id_t;
#define NASD_THREAD_ID_FMT  NASD_64u_FMT
#define NASD_THREAD_ID_NULL ((nasd_thread_id_t)THREAD_NULL)

#define nasd_thread_self() ((nasd_thread_id_t)current_thread())

#define NASD_THREAD_KILL_SELF() { \
  thread_terminate(current_thread()); \
  thread_halt_self(); \
}

typedef struct nasd_sys_threadattr_s  nasd_sys_threadattr_t;
struct nasd_sys_threadattr_s {
  int  dummy;
};

typedef thread_t  nasd_sys_thread_t;
typedef void *    nasd_threadarg_t;

#if NASD_USE_DUX_RW_LOCKS > 0

#define NASD_TS_RWLOCK_DECLARED 1

#define NASD_DECLARE_RWLOCK(_name_) lock_data_t _name_ ;
#define NASD_LOCK_READ(_l_)     lock_read(&(_l_))
#define NASD_UNLOCK_READ(_l_)   lock_read_done(&(_l_))
#define NASD_LOCK_WRITE(_l_)    lock_write(&(_l_))
#define NASD_UNLOCK_WRITE(_l_)  lock_write_done(&(_l_))

#endif /* NASD_USE_DUX_RW_LOCKS > 0 */

#endif /* !_NASD__NASD_DUX_THREADS_KERNEL_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
