/*
 * nasd_nfs_client_dux.h
 *
 * Types for NASD in the kernel (but not needed by DCE)
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Erik Riedel
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 *
 * Network-Attached Secure Disks (NASD) Project, Parallel Data Laboratory
 * Carnegie Mellon University, http://www.pdl.cs.cmu.edu/NASD/
 *
 */

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_nfs_types.h>
#include <nasd/nasd_pdrive_client.h> /* for nasd_drive_handle_t */
#include <dce/rpc.h>                 /* for rpc_binding_handle_t */

/* to keep track of binding handles on the client */

/* NOTE: the size of this structure must be less than what is set in 
	nfs/nfs_clnt.h for struct mntinfo mi_nasd_handles */

typedef struct nasd_kernel_handles_s {
  nasd_nfs_handle_t         server_handle;
  nasd_drive_handle_t       drive_handle;
  nasd_int32                flags;
  nasd_int32                partnum;
} nasd_kernel_handles_t;

#define NASD_HANDLE_NEWDRIVE(_h_) ((_h_)->flags&NASD_BINDING_DRIVE_NEW)

#define NASD_BINDING_SERVER_VALID      0x00000001
#define NASD_BINDING_DRIVE_VALID       0x00000002
#define NASD_BINDING_DRIVE_NEW         0x00000004

/* what we are going to put into the NFS3 filehandle */

/* The size of this structure _must_ be <= NFS3_FHSIZE (64) */

/* The first 32 bytes must be filesystem-wide unique and must
   be identical for identical objects (i.e. one-to-one correspondence) */
/* What we are going to do is stuff this into an nfs3 filehandle, then
   we are going to change the filehandle comparison code to only look
   at the first NFS_FHSIZE (32) bytes when doing compares.  This allows
   us to put our cookie bits into the other 32 bytes. */

typedef struct nasd_identpair_s {
  nasd_nfs_identifier_t identifier; 	/* 16 bytes */
  nasd_int32    pad1[4];		/* 16 bytes */
  nasd_credential_t cred;               /* 8  bytes */
  nasd_int32    pad2[6];                /* 24 bytes */
} nasd_identpair_t;			/* 64 bytes */

/*
 * !!! Match with aprintfidentpair()
 */
#define INIT_NASD_NFS_IDENTIFIER(_id_) { \
  (_id_).pad1 = 0xfad0eaea; \
}


/*
 * !!! Match with aprintfidentpair()
 */
#define INIT_NASD_IDENTPAIR_PAD(_idp_) { \
  (_idp_).pad1[0] = 0x4e415344; \
  (_idp_).pad1[1] = 0x444e4153; \
  (_idp_).pad1[2] = 0x53444e41; \
  (_idp_).pad1[3] = 0x4153444e; \
  (_idp_).pad2[0] = 0x12345678; \
  (_idp_).pad2[1] = 0x9abcdef0; \
  (_idp_).pad2[2] = 0xffffffff; \
  (_idp_).pad2[3] = 0xfeedfeed; \
  (_idp_).pad2[4] = 0xabababab; \
  (_idp_).pad2[5] = 0xcdcdcdcd; \
  INIT_NASD_NFS_IDENTIFIER((_idp_).identifier); \
}

/* backwards compat */
#define INIT_PAD(_id_) INIT_NASD_IDENTPAIR_PAD(_id_)

#define NASD_NFS_MAX_ACTIVE_CAPS     1024
#define NASD_NFS_CAP_TABLE_SZ        128

typedef struct nasd_nfs_captable_entry_s nasd_nfs_captable_entry_t;

struct nasd_nfs_captable_entry_s {
  nasd_nfs_identifier_t       identifier; /*  16 bytes */ 
  nasd_credential_t           cred;       /*   8 bytes */
  nasd_cookie_t               cookie;     /*  56 bytes */
  nasd_nfs_captable_entry_t  *next;       /*   8 bytes */
  nasd_nfs_captable_entry_t  *prev;       /*   8 bytes */
  nasd_nfs_captable_entry_t  *l_next;     /*   8 bytes */
  nasd_nfs_captable_entry_t  *l_prev;     /*   8 bytes */
};                                        /* 112 bytes total */

#define NASD_NFS_IDPAIR_HASH_CALC(idp) ((  (idp)->identifier.nasd_identifier + (idp)->identifier.disk_identifier+(idp)->identifier.partnum << 4) % NASD_NFS_CAP_TABLE_SZ )

/* This looks ugly but any decent compiler will to the right thing */
#define NASD_NFS_IDPAIR_HASH(idp) ( (idp)->identifier.nasd_identifier > 0 ? ((idp)->identifier.nasd_identifier % NASD_NFS_CAP_TABLE_SZ) : ((-1 * (idp)->identifier.nasd_identifier) % NASD_NFS_CAP_TABLE_SZ) ) 

#define SETUP_SECURITY_PARAM(_arg_) { \
    (_arg_).in_security_param.type=(_arg_).in_capability_args.type; \
    (_arg_).in_security_param.partnum=(_arg_).in_capability_args.partnum; \
    (_arg_).in_security_param.actual_protection=(_arg_).in_capability_args.min_protection; \
}


nasd_status_t nasd_nfs_caps_lookup(nasd_identpair_t *idpair, nasd_cookie_t *cookie);
nasd_status_t nasd_nfs_caps_init();
nasd_status_t nasd_nfs_caps_insert(nasd_identpair_t *idpair, nasd_cookie_t *cookie);
