/*
 * mpdrspeed_common.h
 *
 * Shared info for mpdrspeed test
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Authors: David Rochberg, Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__MPDRSPEED_COMMON_H_
#define _NASD__MPDRSPEED_COMMON_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_types.h>
#include <sys/time.h>

#ifdef NSEC_PER_SEC
#undef NSEC_PER_SEC
#endif /* NSEC_PER_SEC */
#define NSEC_PER_SEC 1000000000

#ifndef USEC_PER_SEC
#define USEC_PER_SEC (1000000)
#endif /* USEC_PER_SEC */


#ifndef timespec_sub
#define timespec_sub(result, subtrahend) {          \
	if ((result).tv_nsec >= (subtrahend).tv_nsec) { \
		(result).tv_nsec -= (subtrahend).tv_nsec;   \
		(result).tv_sec -= (subtrahend).tv_sec;     \
	}                                               \
	else {                                          \
		(result).tv_nsec += NSEC_PER_SEC;           \
		(result).tv_nsec -= (subtrahend).tv_nsec;   \
		(result).tv_sec -= (subtrahend).tv_sec + 1; \
	}                                               \
}
#endif /* !timespec_sub */

#ifndef timeval_sub
#define timeval_sub(result, subtrahend) {          \
	if ((result).tv_usec >= (subtrahend).tv_usec) { \
		(result).tv_usec -= (subtrahend).tv_usec;   \
		(result).tv_sec -= (subtrahend).tv_sec;     \
	}                                               \
	else {                                          \
		(result).tv_usec += USEC_PER_SEC;           \
		(result).tv_usec -= (subtrahend).tv_usec;   \
		(result).tv_sec -= (subtrahend).tv_sec + 1; \
	}                                               \
}
#endif /* !timeval_sub */



#define MULTI_BUF_SIZE (1024)
#define MPDRSPEED_MULTI_MAGIC ("mpdrspeedobjects")
#define MPDRSPEED_MAX_OBJS (64)
typedef struct mpdrspeed_header_s {
  char			magic[16];
  nasd_uint64		nobjects;
  nasd_identifier_t	objects[MPDRSPEED_MAX_OBJS];
} mpdrspeed_header_t;
  

#endif /* !_NASD__MPDRSPEED_COMMON_H_ */
