/*
 * nasd_cheops_locks.h
 */
/*
 * Lock module header file, implements multi-reader single-writer locks
 *
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Khalil Amiri, CMU SCS/ECE, July 18 1997
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD_CHEOPS_LOCKS_H_
#define _NASD_CHEOPS_LOCKS_H_

#include <dce/rpc.h>
#include <stdio.h>
#include <malloc.h>

#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_cheops_types.h>
#include <nasd/nasd_threadstuff.h>

typedef struct _nasd_cheops_rwlock_s {
        NASD_DECLARE_MUTEX(m)
        NASD_DECLARE_COND(canRead)
        NASD_DECLARE_COND(canWrite)
        int  nActive;
        int  nPendingReaders;
        int  nPendingWriters;
} _nasd_cheops_rwlock_t;

int 
_nasd_cheops_lockInit( _nasd_cheops_rwlock_t *l );

int 
_nasd_cheops_lockShared( _nasd_cheops_rwlock_t *l );

int 
_nasd_cheops_unlockShared( _nasd_cheops_rwlock_t *l);

int 
_nasd_cheops_lockExclusive( _nasd_cheops_rwlock_t *l );

int 
_nasd_cheops_unlockExclusive( _nasd_cheops_rwlock_t *l );

int 
_nasd_cheops_downgrade( _nasd_cheops_rwlock_t *l );

int 
_nasd_cheops_upgrade( _nasd_cheops_rwlock_t *l );


#endif /* !_NASD_CHEOPS_LOCKS_H_ */


