/*
 * nasd_drive_options.h
 *
 * drive-specific nasd_options.h
 */
/*
 * Copyright (c) 1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_DRIVE_OPTIONS_H_
#define _NASD__NASD_DRIVE_OPTIONS_H_

#include <nasd/nasd_options.h>


/*
 * Chunks of code to include/not include
 */


/* keep basic I/O subsys stats */
#define NASD_IO_KEEP_STATS 1

/* tracing in drive */
#define NASD_TRACE_DR 0

#if (NASD_SECURE_RPCS_ENABLE > 0)
/* Use key cache */
#define  NASD_USE_KEY_CACHE 1
#endif /* NASD_SECURE_RPCS_ENABLE  > 0*/

/* when using region-based layout, activate scoreboard */
#define NASD_NL_REG_SCOREBOARD 0

/* determine expense of maintianing region scoreboard */
#define NASD_NL_REG_SCOREBOARD_TIMERS 0

/* extended direct pointers in the drive */
#if NASD_SECURE_RPCS_ENABLE > 0
#define NASD_OD_EXT_PTR 1
#else /* NASD_SECURE_RPCS_ENABLE > 0 */
#define NASD_OD_EXT_PTR 0
#endif /* NASD_SECURE_RPCS_ENABLE > 0 */

/* keep timers for I/O subsys */
#define NASD_IO_TIMERS 1

/* keep timers for ioqueue subsys */
#define NASD_IOQUEUE_TIMERS 1

/* disable concurrent DCE pipe operations */
#define NASD_DRIVE_DISABLE_CONCURRENT_DCE_PIPES 1




/*
 * Compile-time policy decisions
 */


/*
 * 0 = simple lru
 * 1 = multiple chances for metadata
 * 2 = multiple LRUs
 */
#define NASD_DRIVE_BLOCK_REPLACEMENT 1

/* wire refcnt blocks */
#define NASD_DRIVE_WIRE_REFCNT 1

/* preread refcnts into cache at startup */
#define NASD_DRIVE_PREREAD_REFCNT 1

/*
 * 0 = NPT blocks swappable
 * 1 = NPT1 blocks swappable
 * 2 = NPT not swappable
 */
#define NASD_DRIVE_WIRE_NPT 1

/* preread NPT into cache at startup */
#define NASD_DRIVE_PREREAD_NPT 0

/*
 * If zero, try to create inode in same hash table
 * block as last create. If nonzero, do it in the one
 * after that.
 */
#define NASD_DRIVE_ROTATE_CREATE 0

/*
 * 0 = allow nasd_odc_block_get_part2() to do all
 *     clustering when fetching inode blocks
 *     in nasd_odc_node_get()
 * 1 = always fully cluster I/Os within nasd_odc_node_get()
 * 2 = fully cluster unless inode is first block
 */
#define NASD_DRIVE_INODE_FETCH_CLUSTER_POLICY 2




/*
 * Debugging options
 */


/* keep track of enqueue file and line */
#define NASD_DRIVE_TRACK_IO_ENQUEUE_INFO 0

/* print info about system shutting down */
#define NASD_DRIVE_SHUTDOWN_DEBUG 1

/* track cache block release */
#define NASD_DRIVE_DEBUG_RELEASE 0

/* debug I/O lock */
#define NASD_IO_LOCK_DEBUG 0

/*
 * I/O module should keep debugging list of outstanding I/O.
 * Currently only supported by nasd_od_kio.c
 */
#define NASD_DRIVE_DEBUG_PHYS_OUTSTANDING 0




/*
 * Pluggable modules to include/not include
 */

/*
 * Disk queueing (be sure to include the default!)
 */
#define NASD_DRIVE_IOQUEUE_INCLUDE_CSCAN  1
#define NASD_DRIVE_IOQUEUE_INCLUDE_FIFO   1

/*
 * Layout (be sure to include the default!)
 */
#define NASD_DRIVE_LAYOUT_SEQUENTIAL_INCLUDE  1
#define NASD_DRIVE_LAYOUT_REGION_INCLUDE      0


#endif /* !_NASD__NASD_DRIVE_OPTIONS_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
