/*
 * nasd_nfs_client.h
 *
 * Include for NASD-NFS clients.
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_NFS_CLIENT_H_
#define _NASD__NASD_NFS_CLIENT_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_common.h>
#include <nasd/nasd_shutdown.h>
#include <nasd/nasd_pdrive_client.h>
#include <nasd/nasd_nfs_server.h>

/*
 * La la la. I am lazy.
 */
typedef nasd_drive_handle_t           nasd_nfs_handle_t;
typedef nasd_drive_param_dce_kpdev_t  nasd_nfs_param_dce_kpdev_t;

/*
 * init/shutdown
 */
nasd_status_t nasd_nfscli_real_init(void);
void nasd_nfscli_real_shutdown(void);
void nasd_nfscli_sys_init(void);
nasd_status_t nasd_nfscli_init(void);
void nasd_nfscli_shutdown(void);
void nasd_nfscli_shutdown_drivecli(void *ignored);

/*
 * handle management
 */
extern nasd_status_t nasd_bind_to_nfs_server(char *servername, char *portnum,
  int binding_type, void *binding_param, int binding_param_len,
  nasd_nfs_handle_t *handle);

extern nasd_status_t nasd_unbind_nfs_server(nasd_nfs_handle_t *handle);

/*
 * RPC wrappers
 */

extern void nasd_nfscli_null(nasd_nfs_handle_t handle,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_mount(nasd_nfs_handle_t handle,
  nasd_credential_t in_credential, nasd_string_t dirpath[],
  nasd_list_len_t in_listlen, nasd_net_address_t out_drivelist[],
  nasd_list_len_t *out_listlen, nasd_cookie_t *out_cookie,
  nasd_nfs_identifier_t *out_identifier,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_fsstat(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_fsstat_t *out_stat,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_fsinfo(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_fsinfo_t *out_info,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_lookup_otw_provided(nasd_nfs_handle_t handle,
  nasd_nfs_lookup_args_t *args, nasd_nfs_lookup_res_t *res,
  nasd_byte_t *args_otw_p, nasd_byte_t *res_otw_p,
  nasd_rpc_status_t *status);

extern void nasd_nfscli_lookup(nasd_nfs_handle_t handle,
  nasd_nfs_lookup_args_t *args, nasd_nfs_lookup_res_t *res,
  nasd_rpc_status_t *status);

extern void nasd_nfscli_readdir(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_marker_t in_marker,
  nasd_markerverf_t in_markerverf, nasd_len_t in_count,
  nasd_marker_t *out_marker, nasd_markerverf_t *out_markerverf,
  nasd_len_t *out_count, nasd_boolean_t *out_eof,
  nasd_dirent_t out_entries[], nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_access(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_access_t in_access,
  nasd_access_t *out_access, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_setattr(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_attribute_t in_attribute,
  nasd_fieldmask_t in_fieldmask, nasd_guard_t in_guard,
  nasd_attribute_t *out_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_create(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_attribute_t in_attribute, nasd_fieldmask_t in_fieldmask,
  nasd_createmask_t in_createmask, nasd_cookie_t *out_cookie,
  nasd_nfs_identifier_t *out_identifier, nasd_attribute_t *out_attribute,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_symlink(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_string_t in_symlink[], nasd_attribute_t in_attribute,
  nasd_cookie_t *out_cookie, nasd_nfs_identifier_t *out_identifier,
  nasd_attribute_t *out_attribute, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_remove(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_mkdir(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_attribute_t in_attribute, nasd_fieldmask_t in_fieldmask,
  nasd_cookie_t *out_cookie, nasd_nfs_identifier_t *out_identifier,
  nasd_attribute_t *out_attribute, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_rmdir(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_newcookie(nasd_nfs_handle_t handle,
  nasd_nfs_identifier_t in_directory, nasd_credential_t in_credential,
  nasd_cookie_t *out_cookie,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_rename(nasd_nfs_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_from_directory,
  nasd_credential_t in_credential, nasd_string_t in_from_path[],
  nasd_nfs_identifier_t in_to_directory, nasd_string_t in_to_path[],
  nasd_post_attribute_t *post_from_attribute,
  nasd_post_attribute_t *post_to_attribute,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

extern void nasd_nfscli_getstats(nasd_nfs_handle_t handle,
  nasd_nfs_opstats_t *opstats, nasd_nfs_cachestats_t *cachestats,
  nasd_nfs_rpc_opdepths_t *opdepths,
  nasd_status_t *nasd_status, nasd_rpc_status_t *status);

/*
 * DCE-direct wrappers
 */

extern void nasd_nfsdce_null(rpc_binding_handle_t handle,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_mount(rpc_binding_handle_t handle,
  nasd_credential_t in_credential, nasd_string_t dirpath[],
  nasd_list_len_t in_listlen, nasd_net_address_t out_drivelist[],
  nasd_list_len_t *out_listlen, nasd_cookie_t *out_cookie,
  nasd_nfs_identifier_t *out_identifier,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_fsstat(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_fsstat_t *out_stat,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_fsinfo(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_fsinfo_t *out_info,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_lookup_otw_provided(rpc_binding_handle_t handle,
  nasd_nfs_lookup_args_t *args, nasd_nfs_lookup_res_t *res,
  nasd_byte_t *args_otw_p, nasd_byte_t *res_otw_p,
  error_status_t *status);

extern void nasd_nfsdce_lookup(rpc_binding_handle_t handle,
  nasd_nfs_lookup_args_t *args, nasd_nfs_lookup_res_t *res,
  error_status_t *status);

extern void nasd_nfsdce_readdir(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_marker_t in_marker,
  nasd_markerverf_t in_markerverf, nasd_len_t in_count,
  nasd_marker_t *out_marker, nasd_markerverf_t *out_markerverf,
  nasd_len_t *out_count, nasd_boolean_t *out_eof,
  nasd_dirent_t out_entries[], nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_access(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_access_t in_access,
  nasd_access_t *out_access, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_setattr(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_identifier,
  nasd_credential_t in_credential, nasd_attribute_t in_attribute,
  nasd_fieldmask_t in_fieldmask, nasd_guard_t in_guard,
  nasd_attribute_t *out_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_create(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_attribute_t in_attribute, nasd_fieldmask_t in_fieldmask,
  nasd_createmask_t in_createmask, nasd_cookie_t *out_cookie,
  nasd_nfs_identifier_t *out_identifier, nasd_attribute_t *out_attribute,
  nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_symlink(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_string_t in_symlink[], nasd_attribute_t in_attribute,
  nasd_cookie_t *out_cookie, nasd_nfs_identifier_t *out_identifier,
  nasd_attribute_t *out_attribute, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_remove(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_mkdir(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_attribute_t in_attribute, nasd_fieldmask_t in_fieldmask,
  nasd_cookie_t *out_cookie, nasd_nfs_identifier_t *out_identifier,
  nasd_attribute_t *out_attribute, nasd_post_attribute_t *post_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_rmdir(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_directory,
  nasd_credential_t in_credential, nasd_string_t in_dirpath[],
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_newcookie(rpc_binding_handle_t handle,
  nasd_nfs_identifier_t in_directory, nasd_credential_t in_credential,
  nasd_cookie_t *out_cookie,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_rename(rpc_binding_handle_t handle,
  nasd_cookie_t in_cookie, nasd_nfs_identifier_t in_from_directory,
  nasd_credential_t in_credential, nasd_string_t in_from_path[],
  nasd_nfs_identifier_t in_to_directory, nasd_string_t in_to_path[],
  nasd_post_attribute_t *post_from_attribute,
  nasd_post_attribute_t *post_to_attribute,
  nasd_status_t *nasd_status, error_status_t *status);

extern void nasd_nfsdce_getstats(rpc_binding_handle_t handle,
  nasd_nfs_opstats_t *opstats, nasd_nfs_cachestats_t *cachestats,
  nasd_nfs_rpc_opdepths_t *opdepths,
  nasd_status_t *nasd_status, error_status_t *status);

extern nasd_status_t nasd_nfscli_dce_init(nasd_shutdown_list_t *sl);

#ifdef DEC_OSF
#include <nasd/dux/nasd_nfs_client_dux.h>
#endif /* DEC_OSF */

#endif /* !_NASD__NASD_NFS_CLIENT_H_ */
