/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Author: Howard Gobioff
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/***********************************************************************
 *
 * nasd_nonce_mgmt.h
 *
 * Handle checking if a nonce is fresh or resued
 ***********************************************************************/
#ifndef _NASD_NONCE_MGMT_H
#define _NASD_NONCE_MGMT_H

/* Number of nonce buckets we spread this over -
   each bucket costs us one pointer worth of memory

   This is not changed at run time. If you guess too few buckets,
   you will have long chains in the buckets and searching will
   take a long time. If you have too many, you will waste space.
   
   This size should be fine for prototypes where we can get
   roughly 300 reqs/sec. For a total of 12000 possible within
   NIO
*/
#define NASD_NONCE_BUCKETS 100


/* This parameter defines how close to the current time a nonce
must be to be considered acceptable. */

#define NASD_SKEW_SECS nasd_int64cast(3000)
#define NASD_ACCEPTABLE_SKEW (NASD_NSEC_PER_SEC*NASD_SKEW_SECS)

#include <nasd/nasd_types.h>
#include <nasd/nasd_general.h>
#include <nasd/nasd_shutdown.h>

#define NONCE_HASH(el) (((el)->time>>8)+(el)->ni)


/* Actually we should key off of the capability key */
typedef struct nasd_nonce_el_s {
  nasd_uint64 time;
  nasd_identifier_t ni;
  nasd_int32 expiration;
  nasd_drive_name_t drive_name;
  struct nasd_nonce_el_s *next;
} nasd_nonce_el_t;
/* Since we don't have the entire cookie, we could get a false positive
match but this is unlikely */



nasd_status_t nasd_init_nonce_mgmt(nasd_shutdown_list_t *sl);

nasd_status_t nasd_check_nonce(nasd_capability_args_t *cap_args,
			       nasd_timespec_t *nonce);

void nasd_cl_shutdown_nonce_table(void *ignored);

#endif /* !_NASD_NONCE_MGMT_H */
