/*
 * Copyright (c) 1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

/***********************************************************************
 *
 * nasd_security.h
 *
 * Put the Security in NASD!
 *
 ***********************************************************************/
#ifndef _NASD_SECURITY_H_
#define _NASD_SECURITY_H_ 
#include <nasd/nasd_security_types.h>
#include <nasd/nasd_itypes.h>

/* Default "name" for all NASD drives. This name is used when a drive
   starts up if none is given as a startup argument */
#define NASD_DIGEST_BLOCK_SIZE 32768
#define NASD_OD_BLOCK_SIZE 8192
#define DEFAULT_DRIVE_NAME 0
 

/* This macro takes the _args_ pointer of type _type_ and initializes
the initial vector(IV) for encryption with some function of the _args */

#define SETUP_IV(_type_, _args_, _icv_) \
    bcopy((char *)&(((_type_ *) _args_ )->in_digest.nonce),(char *) _icv_, NASD_IV_SIZE)

/* This macro takes a current IV and transforms it to be the IV to the reply.
This is a deterministic and publically known transformation. The important 
part is that it be DIFFERENT from the request IV and that it be unlikely
that you find a request or reply under the same key with the same IV */
#define SETUP_REPLY_IV(_icv_)  (_icv_)[0]=~((_icv_)[0])

     
     /* in client/nasd_security_rpc.c */
     
nasd_status_t nasd_security_digest_args(nasd_args_t *in_args,
                                        nasd_key_t in_key,
					nasd_byte_t *out_args_otw,
					nasd_args_len_t out_args_len,
                                        nasd_digest_nonce_otw_t in_digest,
                                        nasd_security_context_t* contextp);

nasd_status_t nasd_security_encrypt_args(nasd_args_t *in_args,
                                         nasd_key_t in_key,
                                         nasd_byte_t *icv,
                                         nasd_byte_t *out_args_otw,
                                         nasd_args_len_t out_args_len,
                                         nasd_security_context_t *contextp);


nasd_status_t nasd_security_verify_res(nasd_args_t *in_args,
                                       nasd_key_t in_key,
				       nasd_byte_t *in_res_otw,
				       nasd_args_len_t in_res_len,
				       nasd_status_t *result,
                                       nasd_digest_nonce_otw_t in_digest,
                                       nasd_security_context_t* contextp);


nasd_status_t nasd_security_decrypt_res(nasd_args_t *in_args,
                                        nasd_key_t in_key,
                                        nasd_byte_t *icv,
                                        nasd_byte_t *in_res_otw,
                                        nasd_args_len_t in_res_len,
                                        nasd_security_context_t* contextp);

void nasd_security_reply_init(nasd_args_t *in_args,
                              nasd_key_t in_key,
                              nasd_byte_t* icv,
                              nasd_security_context_t *sctx);

void nasd_security_context_update_receive(nasd_security_context_t *context,
                                          nasd_byte_t *buf, nasd_len_t len,
                                          nasd_offset_t curr);




/* in common/nasd_password.c */
nasd_status_t nasd_security_password_to_keys(char *password,
					     nasd_key_t master_key,
					     nasd_key_t drive_key,
					     nasd_partnum_t partnum,
					     nasd_key_t part_key,
					     nasd_key_t red_key,
					     nasd_key_t black_key);

#if NASD_SECURITY_PRINT_STATS > 0
nasd_status_t nasd_3DES_init_stats();
nasd_status_t nasd_SHA1_init_stats();
nasd_status_t nasd_HMAC_init_stats();

nasd_status_t nasd_3DES_zero_stats();
nasd_status_t nasd_SHA1_zero_stats();
nasd_status_t nasd_HMAC_zero_stats();

void nasd_HMAC_print_stats();
void nasd_SHA1_print_stats();
void nasd_3DES_print_stats();
#endif /* NASD_SECURITY_PRINT_STATS > 0 */

nasd_status_t nasd_security_generate_privacy_key(nasd_key_t cred_key,
                                                   nasd_key_t out_key);

nasd_status_t nasd_security_generate_integrity_key(nasd_key_t cred_key,
                                                   nasd_key_t out_key);

#endif /* !_NASD_SECURITY_H_ */
