/*
 * nasd_shell_internal.h
 *
 * NASD-shell definitions internal to shell core
 */
/*
 * Copyright (c) 1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#ifndef _NASD__NASD_SHELL_INTERNAL_H_
#define _NASD__NASD_SHELL_INTERNAL_H_

#include <nasd/nasd_options.h>
#include <nasd/nasd_shell.h>

typedef struct nasd_shell_hash_s          nasd_shell_hash_t;
typedef struct nasd_shell_hash_ent_s      nasd_shell_hash_ent_t;
typedef struct nasd_shell_hashkey_rock_s  nasd_shell_hashkey_rock_t;
typedef struct nasd_shell_module_s        nasd_shell_module_t;

extern nasd_shutdown_list_t *nasd_shell_shutdown_list;

#define NASD_SHELL_HASH_KEY_BOGUS    0
#define NASD_SHELL_HASH_KEY_STATIC   1
#define NASD_SHELL_HASH_KEY_DYNAMIC  2
#define NASD_SHELL_HASH_KEY_ROCKED   3

#define NASD_SHELL_HASH_KEY_ROCK_LEN 128

struct nasd_shell_hashkey_rock_s {
  char                        key[NASD_SHELL_HASH_KEY_ROCK_LEN];
  nasd_shell_hashkey_rock_t  *next;
};

struct nasd_shell_hash_ent_s {
  char                       *key;
  int                         key_type;
  int                         key_len;
  nasd_shell_hashkey_rock_t  *key_rock;
  void                       *data;
  nasd_shell_hash_ent_t      *next;
  nasd_shell_hash_ent_t      *prev;
};

struct nasd_shell_hash_s {
  int                     nbuckets;
  nasd_shell_hash_ent_t  *buckets;
};

struct nasd_shell_module_s {
  char                 *name;
  nasd_shell_module_t  *hnext;
  nasd_shell_module_t  *hprev;
};

extern char *nasd_shell_prompt;
extern int nasd_shell_prompt_len;

#define NASD_SHELL_PROMPT_DEFAULT "% "

extern void nasd_shell_init_primitives(void);
extern void nasd_shell_init_interactive(void);
extern void nasd_shell_interact(void);

extern void nasd_shell_hash_init(void);
extern void nasd_shell_hash_destroy(nasd_shell_hash_t *hash);
extern nasd_status_t nasd_shell_hash_create(nasd_shell_hash_t **hashp,
  int nbuckets);

#endif /* !_NASD__NASD_SHELL_INTERNAL_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */

