/*
 * kpbind.c
 *
 * Manipulate kpdev bindings
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Jim Zelenka
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_drive_types.h>
#include <nasd/nasd_pdrive_client.h>
#include <nasd/nasd_pdev.h>

char *progname;

extern char *optarg;
extern int   optind;

char *devname = "/dev/nasdkp0";

#define DO_NOTHING 0
#define DO_UNBIND  1

int doit = DO_NOTHING;

void
usage()
{
  fprintf(stderr, "USAGE: %s [options]\n", progname);
  fprintf(stderr, "  -d  devname\n");
  fprintf(stderr, "  -u  unbind\n");
  exit(1);
}

int
main(
  int     argc,
  char  **argv)
{
  int fd, rc;
  char c;

  progname = argv[0];

  while ((c = getopt(argc, argv, "d:u")) != EOF) {
    switch(c) {
      case 'u':
        doit = DO_UNBIND;
        break;
      case 'd':
        devname = optarg;
        break;
      default:
        fprintf(stderr, "Unknown option '%c'\n", c);
        usage();
    }
  }

  if (optind < argc)
    usage();

  fd = open(devname, O_RDWR);
  if (fd < 0) {
    perror("open");
    fprintf(stderr, "ERROR: could not open %s\n", devname);
    exit(1);
  }

  if (doit == DO_UNBIND) {
    rc = ioctl(fd, NASDPD_PUNBIND);
    if (rc) {
      perror("NASDPD_PUNBIND");
      exit(1);
    }
  }

  rc = close(fd);
  if (rc) {
    perror("close");
    exit(1);
  }

  exit(0);
}
