/*
 * nasd_nfs_common.c
 *
 * Common utility functions for NASD/NFS file system prototype.
 */
/*
 * Copyright (c) 1996,1997,1998,1999 Carnegie Mellon University.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <sys/errno.h>
#include <dce/dce_error.h>
#include <dce/rpc.h>

#include <nasd/nasd_types.h>
#include <nasd/nasd_nfs_types.h>


#ifdef _KERNEL

#define NULL 0L
#else /* _KERNEL */
#include <stdio.h>
#endif /* _KERNEL */

/* possible debugging */

/* external utility routines */
extern char* nasd_error_string(nasd_status_t error);

void nasd_nfs_print_attribute (nasd_attribute_t *attribute)
{
  if (attribute) {
    nasd_nfs3_attributes_t *nfs3_specific =
      (nasd_nfs3_attributes_t *)(attribute->fs_specific);

    printf("\n");
    printf("{blk_prealloc(%" NASD_64u_FMT ")",attribute->block_preallocation);
    printf(",blks_used(%" NASD_64u_FMT ")",attribute->blocks_used);
    printf(",blksiz(%u)",attribute->block_size);
    printf(",obj_len(%" NASD_64u_FMT ")",attribute->object_len);
    printf("\n");
    printf(",amtime(%d,%d)",
	    attribute->attr_modify_time.ts_sec,
	    attribute->attr_modify_time.ts_nsec);
    printf(",omtime(%d,%d)",
	    attribute->object_modify_time.ts_sec,
	    attribute->object_modify_time.ts_nsec);
    printf("\n");
    printf(",fsamtime(%d,%d)",
	    attribute->fs_attr_modify_time.ts_sec,
	    attribute->fs_attr_modify_time.ts_nsec);
    printf(",fsomtime(%d,%d)",
	    attribute->fs_object_modify_time.ts_sec,
	    attribute->fs_object_modify_time.ts_nsec);
    printf("\n");
    printf(",crtime(%d,%d)",
	    attribute->object_create_time.ts_sec,
	    attribute->object_create_time.ts_nsec);

    printf(",av(%d)",attribute->av);

    printf("\n");
    printf(",fs{");
    printf("type(%x)",nfs3_specific->type);
    printf(",mode(%o)",nfs3_specific->mode);
    printf(",nlink(%d)",nfs3_specific->nlink);
    printf(",uid(%d)",nfs3_specific->uid);
    printf(",gid(%d)",nfs3_specific->gid);
    printf("}");
    printf("\n");
  }    
}

