/*
 *  naive.c -- naive code generator.
 *
 *  Copyright 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include "usc.h"

static void gbits(Frame *f);		/* generate code for bit-move */
static void gbytes(Frame *f);		/* generate code for full-byte move */



/*  gnaive(f, lalign, ralign) -- generate naive code from frame list  */

void gnaive(Frame *f, int lalign, int ralign)
{
    while (f) {					/* for each frame */
	if (f->f_nbytes == 0)
	    gbits(f);				/* generate bit-move */
	else
	    gbytes(f);				/* generate full-bytes move */
	f = f->f_next;
    }

}



/*  gbits(frame) -- generate code for one sub-byte frame  */

static void gbits(Frame *f)
{
    char mask[10];

    sprintf(mask, "0x%02X", ((1 << f->f_nbits) - 1) << f->f_dstshift);

    if (f->f_flags & MV_ZERO) {
	/* zero fill */
	out("    dst[%d] &= ~%s;\n", f->f_dstoff, mask);
    } else if (f->f_flags & MV_SIGN) {
	/* fill with sign bit */
	out("    dst[%d] = (dst[%d] & ~%s) | (%b & %s);\n",
	    f->f_dstoff, f->f_dstoff, mask, f, mask);
    } else if (f->f_srcshift <= f->f_dstshift) {
	/* copy bits with left shift or no shift */
	out("    dst[%d] = (dst[%d] & ~%s) | (src[%d]<<%d & %s);\n",
	    f->f_dstoff, f->f_dstoff, mask, f->f_srcoff,
	    f->f_dstshift - f->f_srcshift, mask);
    } else {
	/* copy bits with right shift */
	out("    dst[%d] = (dst[%d] & ~%s) | (src[%d]>>%d & %s);\n",
	    f->f_dstoff, f->f_dstoff, mask, f->f_srcoff,
	    f->f_srcshift - f->f_dstshift, mask);
    }
}



/*  gbytes(frame) -- generate code for one full-bytes frame  */

static void gbytes(Frame *f)
{
    if (f->f_flags != 0)
	out("    memset(dst + %d, %b, %d);\n", f->f_dstoff, f, f->f_nbytes);
    else
	out("    memcpy(dst + %d, src + %d, %d);\n",
	    f->f_dstoff, f->f_srcoff, f->f_nbytes);
}
