%{
/*
 *  lexer.l -- lex/flex directives for USC, Version 2.
 *
 *  Copyright 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include "usc.h"
extern YYSTYPE yylval;
%}

%Start CODE COMMENT EPILOG

%%

<INITIAL>^%%		{ BEGIN CODE;
			  printf("/* %%%% */\n");
			  if (gcode == gnaive)
			      printf("\n#include <string.h>\n");
			}
<CODE>^%%		{ BEGIN EPILOG;
			  printf("\n\n/* %%%% */\n\n#line %d \"%s\"\n",
			      iline, iname); }

<INITIAL,EPILOG>^#[ \t]*[0-9]+.*	{ setsource((char*)yytext, 1); }

<INITIAL,EPILOG>.*	{ fputs((char*)yytext, stdout); }
<INITIAL,EPILOG>\n	{ putchar('\n'); iline++; }

<CODE>"/*"		{ BEGIN COMMENT; }
<COMMENT>"*"+"/"	{ BEGIN CODE; }
<COMMENT>"*"+		{ }		
<COMMENT>[^*\n]*	{ }

[ \t]			{ }
\n			{ iline++; }

"<"			{ return TK_LANGLE; }
">"			{ return TK_RANGLE; }
"{"			{ return TK_LBRACE; }
"}"			{ return TK_RBRACE; }
"["			{ return TK_LBRACKET; }
"]"			{ return TK_RBRACKET; }
"("			{ return TK_LPAREN; }
")"			{ return TK_RPAREN; }
"="			{ return TK_ASSIGN; }
"=:"			{ return TK_ASGPAD; }
";"			{ return TK_SEMI; }
","			{ return TK_COMMA; }
"."			{ return TK_PERIOD; }
"&"			{ return TK_AMPER; }
"+"			{ return TK_PLUS; }
"-"			{ return TK_MINUS; }
"*"			{ return TK_ASTER; }
"/"			{ return TK_SLASH; }
"->"			{ return TK_ARROW; }
":"			{ return TK_COLON; }
".."			{ return TK_THRU; }

[0-9]+			{ sscanf((char*)yytext, "%ld", &yylval.y_long);
			  return TK_ICONST; }

[A-Za-z_][A-Za-z_0-9]*	{ char *name = savename((char*)yytext);
			  Symbol *s = lookup(name, 0);
			  if (s == NULL) {		/* unrecognized ID */
			      yylval.y_string = name;
			      return TK_ID;
			  } else if (s->s_scode == S_KEYWORD) {
			      return s->s_val;		/* language keyword */
			  } else if (s->s_scode == S_TYPEDEF) {
			      yylval.y_type = s->s_type;
			      return TK_TNAME;		/* type name */
			  } else {
			      yylval.y_string = name;
			      return TK_ID;		/* ID seen earlier */
			  }
			}


^#ident.*		{ /* ignore Sun cpp output identification line */ }

^#[ \t]*pragma.*	{ warning("pragma ignored", NULL); }

^#[ \t]*[0-9]+.*	{ setsource((char*)yytext, 0); }


.			{ return TK_ERROR; /* unrecognized character */ }

%%


/*
 *  initkw() -- initialize keywords in symbol table.
 *
 *  Does not include predefined type names; see inittypes().
 */
static struct kywd {char *s; int v;} keywords[] = {
    {"inline",	TK_INLINE },
    {"macro",	TK_MACRO },
    {"mdef",	TK_MDEF },
    {"return",	TK_RETURN },
    {"sizeof",	TK_SIZEOF },
    {"static",	TK_STATIC },
    {"struct",	TK_STRUCT },
    {"typedef",	TK_TYPEDEF },
    {"union",	TK_UNION },
    {0, 0}
};

void initkw(void)
{
    struct kywd *k;
    Symbol *s;

    for (k = keywords; k->s; k++) {
	s = lookup(savename(k->s), S_KEYWORD);
	s->s_val = k->v;
    }
}


/*
 *  yystring() -- return char* pointer to current token
 *
 *  This is used because the defined type of "extern yytext" differs
 *  between flex and lex.
 */

char *yystring()
{
    return (char *) yytext;
}
