/*
 *  tree.c -- parse tree node manipulation.
 *
 *  Copyright 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include "usc.h"

static void pnode(Nodeptr n);		/* print node synopsis on stderr */

static Nodeptr nchain;			/* chain of allocated nodes */



/*  newnode(tok, l, r) -- create new node with given code and children  */

Nodeptr newnode(int tok, Nodeptr l, Nodeptr r)
{
    Nodeptr n = zalloc(sizeof(Node));

    n->n_lno = iline;
    n->n_tok = tok;
    n->n_chain = nchain;
    nchain = n;

    /* do r first, then l, to end up with l's line number if there is one */
    if (r != NULL) {
	n->n_r = r;
	n->n_lno = r->n_lno;
    }
    if (l != NULL) {
	n->n_l = l;
	n->n_lno = l->n_lno;
    }
    return n;
}



/*  freenodes() -- free all currently allocated nodes  */

void freenodes(void)
{
    Nodeptr p, q;

    for (p = nchain; p != NULL; p = q) {
	q = p->n_chain;
	free(p);
    }
    nchain = NULL;
}



/*  pnode(n) -- print node on stderr, for debugging  */

static void pnode(Nodeptr n)
{
    char *s;

    if (!n)
	fprintf(stderr, "NULL");
    else {
	fprintf(stderr, "%s", tokname(n->n_tok));
	if (n->n_tok == TK_ID && n->n_sym != NULL) {
	    s = n->n_sym->s_name;
	    fprintf(stderr, ":%s", s ? s : "");
	}
	fprintf(stderr, "(");
	if (n->n_l)
	    fprintf(stderr, "%s", tokname(n->n_l->n_tok));
	fprintf(stderr, ",");
	if (n->n_r)
	    fprintf(stderr, "%s", tokname(n->n_r->n_tok));
	fprintf(stderr, ")");
	if (n->n_type) {
	    fprintf(stderr, ":");
	    ptype(n->n_type, 0);
	}
    }
}



/*  ptree(n) -- print tree of nodes on stderr, for debugging  */

void ptree(Nodeptr n)
{
    static int indent = 0;
    int i;

    if (dbflags[DB_LINES])
	fprintf(stderr, "%2d ", n ? n->n_lno : 0);
    else
	fprintf(stderr, "   ");
    for (i = 0; i < indent; i++)
	putc(' ', stderr);
    pnode(n);
    fprintf(stderr, "\n");
    if (n) {
	indent += 3;
	if (n->n_l)
	    ptree(n->n_l);
	if (n->n_r)
	    ptree(n->n_r);
	indent -= 3;
    }
}



#define MAXTOK 500			/* maximum token code expected */
static char *ttable[MAXTOK+1];

char *tokname(int tok)
{
    static char buf[10];
    static int init = 0;		/* initialized ? */

    if (!init) {
#define tname(i,s) if (i < MAXTOK) ttable[i] = s;
#include "tnames.h"
	init = 1;
	}

    if (tok >= 0 && tok <= MAXTOK && ttable[tok])
	return ttable[tok];
    sprintf(buf, "?%d?", tok);
    return buf;
}
