/*
 *  buffer.c -- buffer handling routines.
 *
 *  Copyright 1994, 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include "usc.h"
#include "buffer.h"
#include "symbol.h"
#include "utils.h"

void dump_source (void)
{
  cleanbuf (SourceText);
}

void catbuf (Buffer *one, Buffer *two)
{
  char *indent, *new;
  int temp = BUFFER_SIZE (one), indentlen;

  assert (BUFFER_LEN (one) == strlen (BUFFER_STRING(one)));
  assert (BUFFER_LEN (two) == strlen (BUFFER_STRING(two)));

  if (BUFFER_STRING(two)[0] == '{')
    indent = make_indent (code_stack_depth - 1);
  else
    indent = make_indent (code_stack_depth);
  indentlen = strmem (indent);

  if (BUFFER_LEN (one) + BUFFER_LEN (two) + indentlen >= BUFFER_SIZE (one)) {
    while (BUFFER_LEN (one) + BUFFER_LEN (two) + indentlen >= 
           BUFFER_SIZE (one)) {
      BUFFER_SIZE (one) *= 2;
    }
    new = (char *)Malloc (BUFFER_SIZE (one) * sizeof (char));
    strcpy (new, BUFFER_STRING (one));
    Free (BUFFER_STRING (one), temp);
    BUFFER_STRING(one) = new;
  }
  /* ###### NEED TO PUT THIS BACK IN IN SOME WAY! - it works without it...
     leave well enough alone
  if ((one == func_body || one == func_vars) && 
      (BUFFER_STRING(one)[BUFFER_LEN(one) - 1] == '\n' ||
       BUFFER_STRING(one)[0] == '\0')) {
    strcat (BUFFER_STRING (one), indent);
    BUFFER_LEN (one) += indentlen - 1;
  }
  */
  BUFFER_LEN (one) += BUFFER_LEN (two);
  strcat (BUFFER_STRING (one), BUFFER_STRING (two));
  if (strlen (indent))
    Free (indent, strmem (indent));

  assert (BUFFER_LEN (one) == strlen (BUFFER_STRING(one)));
}

void writebuf (Buffer *buffer, char *fmt, ...)
{
  va_list args;
  static char string[2056];
  char *new, *indent;
  int stringlen;
  int indentlen;
  int temp = BUFFER_SIZE (buffer);

  va_start (args, fmt);

  assert (BUFFER_LEN (buffer) == strlen (BUFFER_STRING (buffer)));

  vsprintf (string, fmt, args);
  stringlen = strlen (string);
  if (stringlen > 2056)
    crash ("Out of memory: %s", string);
  if (string[0] == '{')
    indent = make_indent (code_stack_depth - 1);
  else
    indent = make_indent (code_stack_depth);
  indentlen = strmem (indent);

  if (BUFFER_LEN (buffer) + stringlen + indentlen >= BUFFER_SIZE (buffer)) {
    while (BUFFER_LEN (buffer) + stringlen + indentlen >= BUFFER_SIZE (buffer))
      BUFFER_SIZE (buffer) *= 2;
    new = (char *)Malloc (BUFFER_SIZE (buffer) * sizeof (char));
    strcpy (new, BUFFER_STRING (buffer));
    Free (BUFFER_STRING (buffer), temp);
    BUFFER_STRING (buffer) = new;
  }
  if ((buffer == body1 || 
       buffer == body2 ||
       buffer == body3 ||
       buffer == body4 ||
       buffer == body5 ||
       buffer == vars 
       ) && 
      (BUFFER_STRING(buffer)[0] == '\0' || 
       BUFFER_STRING(buffer)[BUFFER_LEN(buffer) - 1] == '\n')) {
    sprintf (BUFFER_STRING (buffer) + BUFFER_LEN (buffer), "%s", indent);
    BUFFER_LEN (buffer) += indentlen - 1;
  }
  sprintf (BUFFER_STRING (buffer) + BUFFER_LEN (buffer), "%s", string);
  BUFFER_LEN (buffer) += stringlen;
  if (strlen (indent))
    Free (indent, strmem (indent));

  assert (BUFFER_LEN (buffer) == strlen (BUFFER_STRING (buffer)));

  va_end (args);
}

void freebuf (Buffer *buf)
{
  Free (BUFFER_STRING (buf), BUFFER_SIZE(buf));
  Free (buf, sizeof (Buffer));
}

void cleanbuf (Buffer *buffer)
{
  BUFFER_STRING (buffer)[0] = '\0';
  BUFFER_LEN (buffer) = 0;
}

Buffer *makebuf (int size)
{
  Buffer *new = (Buffer *)Malloc (sizeof (Buffer));

  BUFFER_STRING(new) = (char *)Malloc (size * sizeof (char));
  BUFFER_LEN (new) = 0;
  BUFFER_STRING(new)[0] = '\0';
  BUFFER_SIZE(new) = size;
  return new;
}
