/*
 *  debug.h -- debug function prototypes.
 *  if OPTIMIZE is selected, define all functions as empty macros.
 *
 *  Copyright 1994, 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#ifdef OPTIMIZE
#   define DEBUG(a)   if (0)
#else
#   define DEBUG(a)   if ((a) <= Debug_Level)
#endif

#ifdef LINUX
#define dprintf usit_dprintf
#endif /* LINUX */

#ifdef OPTIMIZE

#define dprintf (void)
#define print_constant(a,b)
#define print_sym(a,b,c,d)
#define print_sym_list(a,b,c,d)
#define print_type(a,b,c,d)
#define print_type_list(a,b,c,d)
#define print_type_table(a)
#define print_sym_table(a)
#define print_struct_table(a)
#define print_union_table(a)
#define print_expr(a,b,c)
#define print_type_modifier(a,b,c)

#else

extern void print_type_modifier (char *indent, char *string, TypeData *data);
extern void print_constant (int level, Constant *theconst);
extern void print_sym (int level, int num_tabs, char *string, Symbol *sym);
extern void print_sym_list (int level, int num_tabs, char *string, Symbol *sym);
extern void print_type (int level, int num_tabs, char *string, Type *type);
extern void print_type_list (int level, int num_tabs, char *string, Type *type);
extern void print_type_table (int level);
extern void print_sym_table (int level);

extern void print_struct_table (int level);
extern void print_union_table (int level);
extern void print_expr (int level, int indent, Expr *theexpr);

extern void dprintf(int level, char *fmt, ...);

#endif

extern char *bitorder_to_string (BitInfo *bits);
extern char *byteorder_to_string (ByteOrder border);
extern char *make_indent (int num_tabs);

