%{
/*
 *  types.lex
 *
 *  Copyright 1994, 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include "usc.h"
#include "symbol.h"
#include "debug.h"
#include "buffer.h"
#include "y.tab.h"

extern int yydebug;
int LineNum = 1;
Buffer *SourceText;

void add_text (void)
{
  writebuf (SourceText, (char *)yytext);
}

int id_or_keyword_or_type (char *lex);

%}

let	[_a-zA-Z]
alnum	[_a-zA-Z0-9]
h	[0-9a-fA-F]
o	[0-7]
d	[0-9]
suffix	[UuLl]
white	[ \t\f]
newline [\n\r]

%p 3000


%%

^#[ \t]*[0-9]+.*     { /* ignore cpp line number output */ }
^#ident.*            { /* ignore Sun cpp output identification */ }  

"/*"  {	 
  int i, lasti = 0; dprintf (1000, "COMMENT");
  while ( (i = input()) && i != EOF) {
    if (i == '\n' || i == '\r')
      LineNum++;
    if (i == '/' && lasti == '*')
      break;
    lasti = i;
  }
  if (i == 0 || i == EOF)
    yyerror ("End of file in comment");
}

\"(\\.|[^\"])*\"     { dprintf (1000, "STRING"); add_text(); 
			set_to_string (yylval.theconst, (char *)yytext); 
			return STRING; 
		      }
\"(\\.|[^\"])*[\r\n] {
      	       	     	yyerror ("Adding missing \" to string constant");
                        add_text(); 
			set_to_string (yylval.theconst, (char *)yytext);
			return STRING;
      	       	     }

'.'   	       	     |
'\\.' 	       	     |
'\\{o}({o}{o}?)?'    |
'\\x{h}({h}{h}?)?'   |
0{o}*{suffix}? 	     |
0x{h}+{suffix}?	     |
[1-9]{d}*{suffix}?    { dprintf (1000, "ICON"); add_text(); 
			set_to_int (yylval.theconst, (char *)yytext);
			return ICON; }

({d}+|{d}+\.[^.]{d}*|{d}*\.{d}+)([eE][-+]?{d}+)?[fF]? {
			dprintf (1000, "FCON"); add_text(); 
			set_to_float (yylval.theconst, (char *)yytext);
			return FCON;
      	       	    }

"("   	 { dprintf (1000, "LP"); add_text(); return yytext[0]; }
")"   	 { dprintf (1000, "RP"); add_text(); return yytext[0]; }
"{"   	 { dprintf (1000, "LC"); pushlevel(); add_text(); return yytext[0]; }
"}"   	 { dprintf (1000, "RC"); poplevel(); add_text(); return yytext[0]; }
"["   	 { dprintf (1000, "LB"); add_text(); return yytext[0]; }
"]"   	 { dprintf (1000, "RB"); add_text(); return yytext[0]; }

"->"      { yylval.theop = ARROW; add_text(); return ARROW; }
"..."     { dprintf (1000, "DOTDOTDOT"); add_text(); return DOTDOTDOT;     }
".."      { dprintf (1000, "DOTDOT"); add_text(); return DOTDOT;     }
"."       { yylval.theop = DOT; add_text(); return DOT;     }

"++"  	 { yylval.theop = PLUSPLUS; add_text(); return PLUSPLUS;}
"--"  	 { yylval.theop = MINMIN; add_text(); return MINMIN; }

"~"		 { yylval.theop = TILDE; add_text(); return yytext[0]; }
"!"  	 	 { yylval.theop = BANG; add_text(); return yytext[0]; }

"*"   	 { yylval.theop = STAR; add_text(); return yytext[0]; }
"/"		 { yylval.theop = DIV; add_text(); return yytext[0]; }
"%" 		 { yylval.theop = MOD; add_text(); return yytext[0]; }
"+"   	 { yylval.theop = PLUS; add_text(); return yytext[0]; }
"-"   	 { yylval.theop = MINUS; add_text(); return yytext[0]; }
"<<"		 { yylval.theop = LSHIFT; add_text(); return LSHIFT; }
">>"  	 { yylval.theop = RSHIFT; add_text(); return RSHIFT; }
">="		 { yylval.theop = GE; add_text(); return GE; }
"<="	 	 { yylval.theop = LE; add_text(); return LE; }
"<"		 { yylval.theop = LT; add_text(); return LT; }
">"		 { yylval.theop = GT; add_text(); return GT; }
"!="		 { yylval.theop = NEQ; add_text(); return NEQ; }
"=="		 { yylval.theop = EQEQ; add_text(); return EQEQ; }

"+="     { dprintf (1000, "ADDEQ"); add_text(); return ADDEQ; }
"-="     { dprintf (1000, "SUBEQ"); add_text(); return SUBEQ; }
"*="     { dprintf (1000, "MULEQ"); add_text(); return MULEQ; }
"/="     { dprintf (1000, "DIVEQ"); add_text(); return DIVEQ; }
"%="     { dprintf (1000, "MODEQ"); add_text(); return MODEQ; }
"&="     { dprintf (1000, "ANDEQ"); add_text(); return ANDEQ; }
"|="     { dprintf (1000, "OREQ"); add_text(); return OREQ; }
"^="     { dprintf (1000, "XOREQ"); add_text(); return XOREQ; }
">>="     { dprintf (1000, "RSHIFTEQ"); add_text(); return RSHIFTEQ; }
"<<="     { dprintf (1000, "LSHIFTEQ"); add_text(); return LSHIFTEQ; }

"="   	 { yylval.theop = EQUAL; add_text(); return EQUAL; }
"&"   	 { yylval.theop = AND; add_text(); return AND; }
"^"   	 { yylval.theop = XOR; add_text(); return XOR; }
"|"   	 { yylval.theop = OR; add_text(); return OR; }
"&&"  	 { yylval.theop = ANDAND; add_text(); return ANDAND; }
"||"  	 { yylval.theop = OROR; add_text(); return OROR; }
"?"   	 { dprintf (1000, "QUEST"); add_text(); return yytext[0]; }
":"   	 { dprintf (1000, "COLON"); add_text(); return yytext[0]; }
","   	 { yylval.theop = COMMA; add_text(); return yytext[0]; }
";"   	 { dprintf (1000, "SEMI"); add_text(); return yytext[0]; }

{let}{alnum}*  {  set_to_string (yylval.theconst, (char *)yytext);
                  add_text(); 
		  print_constant (1000, yylval.theconst); 
		  return id_or_keyword_or_type ((char *)yytext); }
{white}+       { add_text(); }
{newline}      { add_text(); LineNum++; }
.     	       yyerror ("Illegal character");
%%

/********************************************************/

typedef struct {
   char *name;
   int val;
} KeyWord;

/* mju: this table is searched with binary search, so it must be sorted */
KeyWord KeyTab[] = {
   { "char",   	  CHAR,	      },
   { "double", 	  DOUBLE,     },
   { "enum",   	  ENUM,	      },
   { "float",  	  FLOAT,      },
   { "int",    	  INT,	      },
   { "long",   	  LONG,	      },
   { "short",  	  SHORT,      },
   { "signed", 	  SIGNED,     },
   { "sizeof", 	  SIZEOF,     },
   { "static", 	  STATIC,     },
   { "struct", 	  STRUCT,     },
   { "typedef",	  TYPEDEF,    },
   { "union",  	  UNION,      },
   { "unsigned",  UNSIGNED,   },
#ifndef __alpha
   { "unusitll",  ULONGLONG,   },
   { "usitll",	  LONGLONG,   },
#endif /* !__alpha */
   { "void",   	  VOID,	      },
};

int cmp (const void *va, const void *vb)
{
  KeyWord *a, *b;
  a = (KeyWord *)va;
  b = (KeyWord *)vb;
  return strcmp (a->name, b->name);
}

int id_or_keyword_or_type (char *lex)
{
  KeyWord *p, dummy;
  Type *t;
  
  dummy.name = lex;
  p = (KeyWord *)bsearch (&dummy, KeyTab, 
			  sizeof (KeyTab) / sizeof (KeyWord),
			  sizeof (KeyWord), 
			  cmp);
  if (p) {
    dprintf (1000, "KEYWORD: %s", p->name);
    if ( ( p->val == SIZEOF )||( p->val == RETURN ) )
      yylval.theop = p->val;
    return p->val;
  }
  else {
    if (t = find_type (lex)) {
      yylval.thetype = t;
      return TYPEDEFNAME;
    }
    else {
      dprintf (1000, "IDENT: %s", lex);
      return IDENT;
    }
  }
}

int yyerror(char *s)
{
  fprintf (stderr, "lex error in line %d: %s: <%s>\n", LineNum, s, yytext);
  return(-1); /* ??? */
}

void inityy(void)
{
# ifdef YYDEBUG
    yydebug = 0;
# endif
  SourceText = makebuf (128);
}

