/*
 *  usc.h -- general include files and useful definitions for usit.
 *
 *  Copyright 1994, 1995 by the Arizona Board of Regents.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that this notice appears in all copies and supporting documentation.
 *  This software is provided "as is", with no warranty.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <time.h>

#ifndef ASSERT
#define NDEBUG
#endif
#include <assert.h>

#include "hash.h"
#include "mem.h"
#include "symdefs.h"
#include "config.h"
#include "exprdefs.h"
#include "debug.h"

#ifndef DEBUGLEVEL
#define DEBUGLEVEL 0
#endif

#ifndef TRIGGER_COREDUMP
#define TRIGGER_COREDUMP 0
#endif


typedef enum {
  GEN_PLAIN = 0x01, GEN_INLINE = 0x02, GEN_MACROS = 0x04, GEN_STATIC = 0x08
} CodeType;

#define CODE_IS_MACRO     (code_type & GEN_MACROS)
#define CODE_IS_INLINE    (code_type & GEN_INLINE)
#define CODE_IS_PLAIN     (code_type & GEN_PLAIN)
#define CODE_IS_STATIC    (code_type & GEN_STATIC)


extern CodeType DefaultCodeType;

extern int Debug_Level;
extern int LineNum;
extern FILE *OutFile, *HeaderFile;
extern char *InFileName;
extern int total_malloced;
extern int print_comments;
extern int preserve_type_names;
extern int define_native_types;
extern int speed_test;
extern CodeType code_type;
extern Hash_Tab *Type_Tab, *Symbol_Tab, *Struct_Defs, *Union_Defs;

extern void inityy(void);
extern void crash (char *fmt, ...);
extern void error (char *fmt, ...);
extern void warning (char *fmt, ...);
extern char *allocate_new_string (char *string);
extern int yyparse (void);
extern int strmem (char *string);
extern int gcd (int x, int y);

#ifndef MAX
#define MAX(a,b)  ((a) >= (b) ? (a) : (b))
#define MIN(a,b)  ((a) <= (b) ? (a) : (b))
#endif

#define LEFT  0
#define RIGHT 1

#ifdef LINUX
extern int yyerror(char *);
extern int yylex(void);
#endif /* LINUX */
