/*
 * dce_err.c
 *
 * Translates a DCE error code to a string, if possible.
 */
/*
 * Copyright (c) 1998,1999 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Marc Unangst
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie
 * Mellon the rights to redistribute these changes.
 */

#include <nasd/nasd_options.h>
#include <stdio.h>
#include <dce/rpc.h>
#include <dce/dce_error.h>

void
usage(void)
{
  fprintf(stderr, "dce_err: usage: dce_err status\n");
}

int
main(int argc, char **argv)
{
  unsigned long status;
  dce_error_string_t err_str;
  int err_status;

  if(argc != 2) {
    usage();
    exit(1);
  }

  status = strtoul(argv[1], (char **) NULL, 0);
  if(status == 0) {
    fprintf(stderr, "dce_err: bad status value\n");
    usage();
    exit(1);
  }

  dce_error_inq_text(status, err_str, &err_status);
  if(err_status) {
    fprintf(stderr, "dce_err: error converting 0x%x: status=0x%x (%s)\n",
	    status, err_status, err_str);
    exit(1);
  }

  printf("status 0x%x is \"%s\"\n", status, err_str);
  exit(0);
}
